==> reduxable-0.4.0.tgz: package/es/Reduxable.js <==
var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import combineReducers from './combineReducers';
import { assertReducersObject, assertState, assertChildName } from './assertions';

import warning from './utils/warning';

function isAReduxableSet(state) {
  if (!state || typeof state !== 'object') {
    return false;
  }

  var firstChild = state[Object.keys(state)[0]];

  if (!firstChild) {
    return false;
  }

  return typeof firstChild === 'function' || firstChild.reduce && typeof firstChild.reduce === 'function';
}

var Reduxable = function () {
  /*
  * The Reduxable constructor receives
  * @param {any} state - Can be an object with other reduxables/reducers or any
  * @param {Object} reducers - An object with pure functions
  */

  function Reduxable() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.constructor.state;
    var reducers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.constructor.reducers;

    _classCallCheck(this, Reduxable);

    assertState(state);

    if (isAReduxableSet(state)) {
      // TODO: assert valid reduxable set
      this._setupChildren(state);
      this.reduce = combineReducers(state);
    } else {
      assertReducersObject(reducers);
      this._setupReducers(reducers);
      this._setupGlobalReducers(this.constructor.globalReducers);
      this._state = state;
      this.reduce = this._getReducer();
    }
  }

  /*
  *  The `_store` will be the Redux store. Since this store is unique by application, then we
  *  can save _statically_ and use it across all the Reduxable instances.
  *
  *  We use this store internally on two Reduxable instance methods:
  *    - `dispatch` to precisely dispatch the actions
  *    - `state` getter to retrieve the portion of state corredpondent to the Reduxable instance
  *
  *  This method is called from `createStore` method. See its documentation for more details.
  */

  Reduxable._setStore = function _setStore(store) {
    this._store = store;
  };

  /*
  *  The `_scope` will define where this Reduxable instance is placed on the global state tree
  *
  *  This scope will be set as a parameter in each dispacthed action and will be used to determine
  *  whether or not an action should be catched by a reducer.
  *
  *  This method is called from `combineReducers` method. See its documentation for more details.
  */

  Reduxable.prototype._setScope = function _setScope(scope) {
    var _this = this;

    this._scope = scope;

    if (this.children) {
      Object.keys(this.children).forEach(function (key) {
        var child = _this.children[key];
        child._setScope && child._setScope(scope + '.' + key);
      });
    }
  };

  /*
  *  `state` getter
  *  ------------
  *  Returns the state for this particular scope
  *
  *  Given the following structure for a Redux store
  *
  *  const reduxStore = createStore(combineReducers({
  *    a: combineReducers({
  *      b: thisReduxableInstance
  *    })
  *  })
  *
  *  Then `thisReduxableInstance.state` will return `reduxStore.state.a.b`
  */

  /*
  *  Returns a reducer function grouping all the defined reducers
  *
  *  This method will do the following
  *    1) Check that the action `scope` match with the scope of this Reduxable instance
  *    2) Find the method that matchs with the action `type`
  *    3) Call that method with the current state and the action `payload`
  *    4) Check that the new state retrieved by that method is not the same
  *       (i.e the method did not mutate the previous state)
  */

  Reduxable.prototype._getReducer = function _getReducer() {
    var _this2 = this;

    return function () {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this2.state;
      var _ref = arguments[1];
      var type = _ref.type;
      var scope = _ref.scope;
      var payload = _ref.payload;

      var globalReducer = _this2._globalReducers[type];
      if (globalReducer) {
        return globalReducer(state, payload);
      }

      if (!_this2.constructor._global && scope !== _this2._scope) {
        return state;
      }

      var scopedReducer = _this2._scopedReducers[type];

      if (scopedReducer) {
        return scopedReducer(state, payload);
      } else {
        // TODO: should we show a warning here? I think this shouldn't be reached never
      }

      return state;
    };
  };

  /*
  *  This method will:
  *  1) store the `reducers`
  *  2) define a method for each reducer that will call the reducer 
  *     See the `_callReducer` method for more info
  *
  */

  Reduxable.prototype._setupReducers = function _setupReducers(reducers) {
    var _this3 = this;

    this._scopedReducers = reducers;
    this.reducers = {};

    var _loop = function _loop(reducerName) {
      if (reducers.hasOwnProperty(reducerName)) {
        // this[reducerName] = payload => this._callReducer(reducerName, payload)
        _this3.reducers[reducerName] = function (payload) {
          return _this3._callReducer(reducerName, payload);
        };
      }
    };

    for (var reducerName in reducers) {
      _loop(reducerName);
    }
  };
  /*
  *  This method will store the `globalReducers` that will listen that actions no matter the scope
  */

  Reduxable.prototype._setupGlobalReducers = function _setupGlobalReducers() {
    var globalReducers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this._globalReducers = globalReducers;
  };

  /*
  *  This method will setup the state as properties
  */


  Reduxable.prototype._setupChildren = function _setupChildren(children) {
    this.children = children;

    for (var childName in children) {
      if (children.hasOwnProperty(childName)) {
        assertChildName(this, childName);
        this[childName] = children[childName];
      }
    }
  };

  /*
  *  This method will _call the reducer_ in two different ways
  *  - If it is connected to Redux, will dispatch an action for that reducer
  *  - If not, will apply the reducer directly an store the new state locally
  *
  */

  Reduxable.prototype._callReducer = function _callReducer(reducerName, payload) {
    var store = this.constructor._store;

    if (store) {
      return store.dispatch({ type: reducerName, scope: this._scope, payload: payload });
    }

    this._state = this._scopedReducers[reducerName](this.state, payload);
  };

  _createClass(Reduxable, [{
    key: 'state',
    get: function get() {
      if (!this.constructor._store) {
        return this._state;
      }
      var rootState = this.constructor._store.getState();
      if (!this._scope) {
        return rootState;
      }

      return this._scope.split('.').reduce(function (object, scopeKey) {
        return object[scopeKey];
      }, rootState);
    }

    /*
    *  `state` setter
    *  ------------
    *  Will set the state if not already set
    */

    ,
    set: function set(newState) {
      warning('You can not set the state directly. You need to call a reducer to mutate the state');
    }
  }]);

  return Reduxable;
}();

export default Reduxable;

==> reduxable-0.4.1.tgz: package/es/Reduxable.js <==
var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

import combineReducers from './combineReducers';
import { assertReducersObject, assertState, assertChildName } from './assertions';

import warning from './utils/warning';

function isAReduxableSet(state) {
  if (!state || typeof state !== 'object') {
    return false;
  }

  var firstChild = state[Object.keys(state)[0]];

  if (!firstChild) {
    return false;
  }

  return typeof firstChild === 'function' || firstChild.__isReduxable;
}

var Reduxable = function () {
  /*
  * The Reduxable constructor receives
  * @param {any} state - Can be an object with other reduxables/reducers or any
  * @param {Object} reducers - An object with pure functions
  */

  function Reduxable() {
    var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : this.constructor.state;
    var reducers = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : this.constructor.reducers;

    _classCallCheck(this, Reduxable);

    this.__isReduxable = true;
    assertState(state);

    if (isAReduxableSet(state)) {
      // TODO: assert valid reduxable set
      this._setupChildren(state);
      this.reduce = combineReducers(state);
    } else {
      assertReducersObject(reducers);
      this._setupReducers(reducers);
      this._setupGlobalReducers(this.constructor.globalReducers);
      this._state = state;
      this.reduce = this._getReducer();
    }
  }

  /*
  *  The `_store` will be the Redux store. Since this store is unique by application, then we
  *  can save _statically_ and use it across all the Reduxable instances.
  *
  *  We use this store internally on two Reduxable instance methods:
  *    - `dispatch` to precisely dispatch the actions
  *    - `state` getter to retrieve the portion of state corredpondent to the Reduxable instance
  *
  *  This method is called from `createStore` method. See its documentation for more details.
  */

  Reduxable._setStore = function _setStore(store) {
    this._store = store;
  };

  /*
  *  The `_scope` will define where this Reduxable instance is placed on the global state tree
  *
  *  This scope will be set as a parameter in each dispacthed action and will be used to determine
  *  whether or not an action should be catched by a reducer.
  *
  *  This method is called from `combineReducers` method. See its documentation for more details.
  */

  Reduxable.prototype._setScope = function _setScope(scope) {
    var _this = this;

    this._scope = scope;

    if (this.children) {
      Object.keys(this.children).forEach(function (key) {
        var child = _this.children[key];
        child._setScope && child._setScope(scope + '.' + key);
      });
    }
  };

  /*
  *  `state` getter
  *  ------------
  *  Returns the state for this particular scope
  *
  *  Given the following structure for a Redux store
  *
  *  const reduxStore = createStore(combineReducers({
  *    a: combineReducers({
  *      b: thisReduxableInstance
  *    })
  *  })
  *
  *  Then `thisReduxableInstance.state` will return `reduxStore.state.a.b`
  */

  /*
  *  Returns a reducer function grouping all the defined reducers
  *
  *  This method will do the following
  *    1) Check that the action `scope` match with the scope of this Reduxable instance
  *    2) Find the method that matchs with the action `type`
  *    3) Call that method with the current state and the action `payload`
  *    4) Check that the new state retrieved by that method is not the same
  *       (i.e the method did not mutate the previous state)
  */

  Reduxable.prototype._getReducer = function _getReducer() {
    var _this2 = this;

    return function () {
      var state = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _this2.state;
      var _ref = arguments[1];
      var type = _ref.type;
      var scope = _ref.scope;
      var payload = _ref.payload;

      var globalReducer = _this2._globalReducers[type];
      if (globalReducer) {
        return globalReducer(state, payload);
      }

      if (!_this2.constructor._global && scope !== _this2._scope) {
        return state;
      }

      var scopedReducer = _this2._scopedReducers[type];

      if (scopedReducer) {
        return scopedReducer(state, payload);
      } else {
        // TODO: should we show a warning here? I think this shouldn't be reached never
      }

      return state;
    };
  };

  /*
  *  This method will:
  *  1) store the `reducers`
  *  2) define a method for each reducer that will call the reducer 
  *     See the `_callReducer` method for more info
  *
  */

  Reduxable.prototype._setupReducers = function _setupReducers(reducers) {
    var _this3 = this;

    this._scopedReducers = reducers;
    this.reducers = {};

    var _loop = function _loop(reducerName) {
      if (reducers.hasOwnProperty(reducerName)) {
        // this[reducerName] = payload => this._callReducer(reducerName, payload)
        _this3.reducers[reducerName] = function (payload) {
          return _this3._callReducer(reducerName, payload);
        };
      }
    };

    for (var reducerName in reducers) {
      _loop(reducerName);
    }
  };
  /*
  *  This method will store the `globalReducers` that will listen that actions no matter the scope
  */

  Reduxable.prototype._setupGlobalReducers = function _setupGlobalReducers() {
    var globalReducers = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    this._globalReducers = globalReducers;
  };

  /*
  *  This method will setup the state as properties
  */


  Reduxable.prototype._setupChildren = function _setupChildren(children) {
    this.children = children;

    for (var childName in children) {
      if (children.hasOwnProperty(childName)) {
        assertChildName(this, childName);
        this[childName] = children[childName];
      }
    }
  };

  /*
  *  This method will _call the reducer_ in two different ways
  *  - If it is connected to Redux, will dispatch an action for that reducer
  *  - If not, will apply the reducer directly an store the new state locally
  *
  */

  Reduxable.prototype._callReducer = function _callReducer(reducerName, payload) {
    var store = this.constructor._store;

    if (store) {
      return store.dispatch({ type: reducerName, scope: this._scope, payload: payload });
    }

    this._state = this._scopedReducers[reducerName](this.state, payload);
  };

  _createClass(Reduxable, [{
    key: 'state',
    get: function get() {
      if (!this.constructor._store) {
        return this._state;
      }
      var rootState = this.constructor._store.getState();
      if (!this._scope) {
        return rootState;
      }

      return this._scope.split('.').reduce(function (object, scopeKey) {
        return object[scopeKey];
      }, rootState);
    }

    /*
    *  `state` setter
    *  ------------
    *  Will set the state if not already set
    */

    ,
    set: function set(newState) {
      warning('You can not set the state directly. You need to call a reducer to mutate the state');
    }
  }]);

  return Reduxable;
}();

export default Reduxable;
